import org.slf4j.Logger
import org.slf4j.LoggerFactory
import org.springframework.http.converter.json.MappingJacksonHttpMessageConverter
import org.springframework.web.client.HttpClientErrorException
import org.springframework.web.client.RestTemplate

import com.sas.jobmonitor.common.spring.CasRestTemplate
import com.sas.svcs.jobmonitor.model.JobExecution
import com.sas.svcs.jobmonitor.model.JobExecutionList
import com.sas.svcs.jobmonitor.model.LogBlock
import com.sas.svcs.jobmonitor.model.Message
import com.sas.svcs.jobmonitor.model.MessageList
import com.sas.svcs.jobmonitor.model.StepExecution
import com.sas.svcs.jobmonitor.model.StepExecutionList


class JobMonitorUtil {

    private Logger log = LoggerFactory.getLogger(this.class)

    private RestTemplate restTemplate
    private String restUrl

    private static JobMonitorUtil instance

    static getInstance() {
        if (!instance)
            instance = new JobMonitorUtil()
        instance
    }
    
    //singleton
    private JobMonitorUtil() {
        def configProvider = new JobMonitorConfigurationProvider()
        
        //use a version of RestTemplate that will handle CAS ticketing for us
        restTemplate = new CasRestTemplate(configProvider)

        //configure RestTemplate to always use JSON for transport
        restTemplate.messageConverters = [new MappingJacksonHttpMessageConverter()]

        restUrl = configProvider.jobMonitorServiceURL ?: ''
        
        //add /rest to the url if it doesn't already have it
        if (!restUrl.endsWith('/rest'))
            restUrl = "$restUrl/rest"

        log.info "JobMonitor midtier service url: $restUrl"
    }
    
    JobExecution[] getFilteredJobExecutions(params) {
        log.debug 'Start fetch data from webservice - getFilteredJobExecutions'
        
        def url = restUrl << '/jobexecutions'
        
        def first = true
        def addParam = { key, value ->
            url << (first ? '?' : '&')
            first = false
            url << "$key=$value"
        }
        
        //filter params
        def after = params.getOne('after')
        def before = params.getOne('before')
        def keyword = params.getOne('keyword')
        def platform = params.getOne('platform')
        def type = params.getOne('type')
        def status = params.getOne('status')
        def userID = params.getOne('userID')
                
        if (after)
            addParam 'after', after
        if (before)
            addParam 'before', before
        if (keyword)
            addParam 'keyword', keyword
        if (platform)
            addParam 'platform', platform
        if (type)
            addParam 'type', type
        if (status)
            addParam 'status', status
        if (userID)
            addParam 'userID', userID
        
        //paging params
        addParam 'limit', 0

        URI uri = fixURI(url)
        log.info('getFilteredJobExecutions uri =' + uri.toString())
        JobExecutionList jobExecs = restTemplate.getForObject(uri, JobExecutionList)
        
        log.debug 'End fetch data from webservice - getFilteredJobExecutions'
        
        jobExecs?.jobExecutions ?: []
    }
    
	URI fixURI(def uri) {
		new URI(uri.toString().replace('+', '%2B'))
	}
	
    JobExecution[] getJobExecutions(params) {                  
       def jobExecID = params.getOne('jobExecID')
       def before = params.getOne('before');
       def after = params.getOne('after');
       
       //need to make sure jobExecID is not undefined
       if (jobExecID == 'undefined') {
          log.info 'Unable to process rest call for undefined job execution'
          return []
       }
        
       def jobExec = getJobExecution(jobExecID.toInteger())
       
       if (before || after)
           getJobExecutions(jobExec.jobID, before, after);
       else
           getJobExecutions(jobExec.jobID)
    }
    
    JobExecution[] getJobExecutions(Integer jobID) {
        log.debug 'Start fetch data from webservice - getJobExecutions'
        
        def url = "$restUrl/jobexecutions?jobID=$jobID&limit=0"
        
        JobExecutionList jobExecs = restTemplate.getForObject(url, JobExecutionList)
                           
        log.debug 'End fetch data from webservice - getJobExecutions'        
        
        jobExecs?.jobExecutions ?: []
    }
    
    JobExecution[] getJobExecutions(Integer jobID, before, after) {
        log.debug 'Start fetch data from webservice - getJobExecutions'
        
        def url = "$restUrl/jobexecutions?jobID=$jobID&limit=0"
        
        if (before) url += "&before=$before"
        if (after)  url += "&after=$after"
        
        URI uri = fixURI(url)
        log.info('getJobExecutions uri =' + uri.toString())
        JobExecutionList jobExecs = restTemplate.getForObject(uri, JobExecutionList)
                           
        log.debug 'End fetch data from webservice - getJobExecutions'
        
        jobExecs?.jobExecutions ?: []
    }

    JobExecution getJobExecution(executionID) {
        log.debug 'Start fetch data from webservice - getJobExecution'
        
        def url = "$restUrl/jobexecutions/$executionID"
        
        JobExecution jobExec = restTemplate.getForObject(url, JobExecution)
        
        log.debug 'End fetch data from webservice - getJobExecution'
        
        jobExec
    }
    
    JobExecution getMostRecentExecutionOfJob(jobID) {
        log.debug 'Start fetch data from webservice - getMostRecentExecutionOfJob'
        
        def url = "$restUrl/jobexecutions/mostrecent?jobID=$jobID"
        
        JobExecution jobExec = restTemplate.getForObject(url, JobExecution)
        
        log.debug 'End fetch data from webservice - getMostRecentExecutionOfJob'
        
        jobExec
    }
    
    StepExecution getStepExecution(stepExecID) {
        log.debug 'Start fetch data from webservice - getStepExecution'
        
        def url = "$restUrl/stepexecutions/$stepExecID"
        
        StepExecution stepExec = restTemplate.getForObject(url, StepExecution)
        
        log.debug 'End fetch data from webservice - getStepExecution'
        
        stepExec
    }
    
    StepExecution[] getStepExecutionChildSteps(params) {                
        def parentStepExecID = params.getOne('parentStepExecID')

        //need to make sure step ID is not undefined
        if (!parentStepExecID) {
            log.info 'Unable to process rest call for undefined job execution'
            return []
        }
        
        getStepExecutionChildSteps(parentStepExecID.toInteger());        
    }
    
    StepExecution[] getStepExecutionChildSteps(Integer stepExecID) {
        getStepExecutionChildSteps(stepExecID, true)
    }
    
    StepExecution[] getStepExecutionChildSteps(Integer stepExecID, boolean bRootLevelOnly) {
        log.debug 'Start fetch data from webservice - getStepExecutionChildSteps'
       
        def url = bRootLevelOnly ? "$restUrl/stepexecutions?parentStepExecID=$stepExecID" : "$restUrl/stepexecutions/$stepExecID/descendants"
        
        StepExecutionList stepExecs = restTemplate.getForObject(url, StepExecutionList)
       
        log.debug 'End fetch data from webservice - getStepExecutionChildSteps'
       
        stepExecs?.stepExecutions ?: []
    }
        
    StepExecution[] getStepsForJobExecution(params) {
        def jobExecID = params.getOne('jobExecID')
        
        //need to make sure jobExecID is not undefined)
        if (!jobExecID) {
            log.info 'Unable to process rest call for undefined job execution'
            return []
        }
        
        getStepsForJobExecution(jobExecID.toInteger())
    }
   
    StepExecution[] getStepsForJobExecution(Integer jobExecID) {
        getStepsForJobExecution(jobExecID, true)
    }
    
    StepExecution[] getStepsForJobExecution(Integer jobExecID, boolean root) {
        log.debug 'Start fetch data from webservice - getStepsForJobExecution'
        
        def url = "$restUrl/stepexecutions?jobExecID=$jobExecID&rootLevelOnly=$root&limit=0"
      
        StepExecutionList stepExecs = restTemplate.getForObject(url.toString(), StepExecutionList)
        
        log.debug 'End fetch data from webservice - getStepsForJobExecution'
        
        stepExecs?.stepExecutions ?: []
    }
    
    StepExecution[] getStepsForJob(jobID, before, after, useBefore, deployedVersion) {
        log.debug 'Start fetch data from webservice for getStepsForJob'
        
        //need to make sure jobID is not undefined)
        if (!jobID) {
            log.info 'Unable to process rest call for undefined job'
            return []
        }
        
        def url = "$restUrl/stepexecutions?jobID=$jobID&deployedVersion=$deployedVersion&after=$after&limit=0&rootLevelOnly=true"
        
        if (useBefore)
            url += "&before=$before"
        
        URI uri = fixURI(url)
        log.info('getStepsForJob uri =' + uri.toString())
        StepExecutionList stepExecs = restTemplate.getForObject(uri, StepExecutionList)
        
        log.debug 'End fetch data from webservice for getStepsForJob'
        
        stepExecs?.stepExecutions ?: []
    }
    
    StepExecution[] getStepExecutions(stepID, iterationIndex, ancestorStepsIterationKey) {
        log.debug 'Start fetch data from webservice for getStepExecutions'
        
        def url = "$restUrl/stepexecutions?stepID=$stepID&limit=0"
        
        if (iterationIndex) url += "&iterationIndex=$iterationIndex"
        if (ancestorStepsIterationKey) url += "&ancestorStepsIterationKey=$ancestorStepsIterationKey"
        
        StepExecutionList stepExecs = restTemplate.getForObject(url.toString(), StepExecutionList)
        
        log.debug 'End fetch data from webservice'
        
        stepExecs?.stepExecutions ?: []
    }
    
    StepExecution[] getFilteredStepExecutions(stepID, iterationIndex, ancestorStepsIterationKey, before, after, useBefore) {
        log.debug 'Start fetch data from webservice for getFilteredStepExecutions'
        
        def url = "$restUrl/stepexecutions?stepID=$stepID&after=$after&limit=0"
        
        if (iterationIndex != null && iterationIndex != "null")
            url += "&iterationIndex=$iterationIndex"
            
        if (ancestorStepsIterationKey != null && ancestorStepsIterationKey != "null")
            url += "&ancestorStepsIterationKey=$ancestorStepsIterationKey"
            
        if (useBefore)
            url += "&before=$before"
        
        URI uri = fixURI(url)
        log.info('getFilteredStepExecutions uri =' + uri.toString())
        StepExecutionList stepExecs = restTemplate.getForObject(uri, StepExecutionList)
        
        log.debug 'End fetch data from webservice'
        
        stepExecs?.stepExecutions ?: []
    }
   
    StepExecution getMostRecentExecutionOfStep(stepID, iterationIndex, ancestorStepsIterationKey) {
        log.debug 'Start fetch data from webservice - getMostRecentExecutionOfStep'
        
        def url = "$restUrl/stepexecutions/mostrecent?stepID=$stepID"
        if (iterationIndex && iterationIndex != 'null') url += "&iterationIndex=$iterationIndex"
        if (ancestorStepsIterationKey && ancestorStepsIterationKey != 'null') url += "&ancestorStepsIterationKey=$ancestorStepsIterationKey"
        
        StepExecution stepExec = restTemplate.getForObject(url, StepExecution)
        
        log.debug 'End fetch data from webservice - getMostRecentExecutionOfStep'
        
        stepExec
    }
    
    StepExecution[] getStepsForStepContainerTrend(parentStepID, ancestorStepsIterationKey, before, after, useBefore) {
        log.debug 'Start fetch data from webservice for getStepsForStepContainerTrend'
        
        def url = "$restUrl/stepexecutions?parentStepID=$parentStepID&after=$after&limit=0"
        
        if (ancestorStepsIterationKey != null && ancestorStepsIterationKey != "null")
            url += "&ancestorStepsIterationKey=$ancestorStepsIterationKey"
            
        if (useBefore)
            url += "&before=$before"
        
        URI uri = fixURI(url)
        log.info('getStepsForStepContainerTrend uri =' + uri.toString())
        StepExecutionList stepExecs = restTemplate.getForObject(uri, StepExecutionList)

        log.debug 'End fetch data from webservice for getStepsForStepContainerTrend'
        
        stepExecs?.stepExecutions ?: []
    }
    
    StepExecution[] getStepHistory(params)
    {
        def stepID = params.getOne('stepID')
        def iterationIndex = params.getOne('iterationIndex')
        def before = params.getOne('before')
        def after = params.getOne('after');
        def ancestorStepsIterationKey = params.getOne('ancestorStepsIterationKey')
        
        if (!stepID) {
            log.info 'Unable to process rest call for missing parameters'
            return []
        }

        return getStepHistory(stepID, iterationIndex, before, after, ancestorStepsIterationKey)
    }
    
    StepExecution[] getStepHistory(stepID, iterationIndex, before, after, ancestorStepsIterationKey) {
        log.debug 'Start fetch data from webservice for getStepHistory'
        
        def url = "$restUrl/stepexecutions?stepID=$stepID&after=$after&limit=0"
        
        if (iterationIndex != null && iterationIndex != "null")
            url += "&iterationIndex=$iterationIndex"
            
        if (ancestorStepsIterationKey != null && ancestorStepsIterationKey != "null")
            url += "&ancestorStepsIterationKey=$ancestorStepsIterationKey"
        
        if (before)
            url += "&before=$before"
        
        URI uri = fixURI(url)
        log.info('getStepHistory uri =' + uri.toString())
        StepExecutionList stepExecs = restTemplate.getForObject(uri, StepExecutionList)

        log.debug 'End fetch data from webservice for getStepHistory'
        
        stepExecs?.stepExecutions ?: []
    }
    
    Message[] getMessagesForJob(jobID) {
        log.debug "Start fetch data from webservice - getMessagesForJob($jobID)"
      
        //need to make sure step ID is not undefined
        if (!jobID) {
            log.info 'Unable to process rest call for undefined job'
            return []
        }
      
        def url = "$restUrl/jobexecutions/$jobID/messages?deep=true"
            
        MessageList messages = restTemplate.getForObject(url.toString(), MessageList)
      
        log.debug "End fetch data from webservice - getMessagesForJob($jobID)"
                 
        messages?.messages ?: []
    }
        
    Message[] getMessagesForStep(stepID) {
        log.debug "Start fetch data from webservice - getMessagesForStep($stepID)"
      
        //need to make sure step ID is not undefined
        if (!stepID) {
            log.info 'Unable to process rest call for undefined step'
            return []
        }
      
        def url = "$restUrl/stepexecutions/$stepID/messages?deep=true"
            
        MessageList messages = restTemplate.getForObject(url.toString(), MessageList)
      
        log.debug "End fetch data from webservice - getMessagesForStep($stepID)"
                 
        messages?.messages ?: []
    }
   
   LogBlock getLogForJobExec(jobExecID) {
       log.debug 'Start fetch data from webservice - getLogForJobExec'
       
       //need to make sure jobExecID is not undefined
       if (!jobExecID) {
          log.info 'Unable to process rest call for undefined job exec'
          return null
       }
       
       def url = "$restUrl/joblogs?executionID=${jobExecID}&limit=0"
       LogBlock logblock
       
       try {
           logblock = restTemplate.getForObject(url.toString(), LogBlock)
       }
       catch(HttpClientErrorException e) {
           log.info "Unable to retrieve log for job execution ID $jobExecID"
           return null
       }
             
       log.debug 'End fetch data from webservice - getLogForJobExec'
       
       logblock
   }
   
   StepExecution[] getAncestorsForStepExecution(stepExecID) {
       log.debug 'Start fetch data from webservice for getAncestorsForStepExec'
       
       def url = "$restUrl/stepexecutions/$stepExecID/ancestors"
       
       StepExecutionList stepExecs = restTemplate.getForObject(url.toString(), StepExecutionList)

       log.debug 'End fetch data from webservice for getAncestorsForStepExec'
       
       stepExecs?.stepExecutions ?: []
   }
}
